/*
** do_charencode.c - written by vesely in milano on 28 sep 99
**
** write charencode.c to be included in mime.c
*/
/*
** Part of MIMEC: see NOTICE.txt
** Copyright (c) 1999 Alessandro Vesely
** All rights reserved.
*/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include "mime.h"

static const char *mime_encoding_desc[] =
{
	"mime_encoding_not_set", /* not set */
	"mime_base_64",
	"mime_quoted_printable",
	"mime_7bit",
	"mime_8bit",
	"mime_binary"
};



int main(int argc, char *argv[])
{
	size_t ch;
	char const *const fname = "charencode.c";
	time_t gentime;
	FILE *fp = fopen(fname, "wt");

	if (fp == NULL)
	{
		perror("do_charencode");
		return 1;
	}
	
	time(&gentime);
	
	fprintf(fp,
		"/*\n"
		"** %s - generated by %s on %s"
		"**\n"
		"** DO NOT EDIT THIS FILE: EDITS WILL BE LOST!\n"
		"*/"
		"\n"
		"#include \"mime.h\"\n"
		"\n"
		"static unsigned char const char_encode[256] = \n"
		"{\n",
			fname, argv[0], ctime(&gentime));
	
	for (ch = 0; ch < 255; ++ch)
	{
		int enctype;
		char buf[120];
		
		if (ch == '\r')
			enctype = mime_encoding_not_set;
		else if (isspace(ch) ||
			ch >= '0' && ch <= '9' ||
			ch >= 'A' && ch <= 'Z' ||
			ch == '+' || ch == '/' || ch == '=')
				enctype = mime_base_64;
		else if (!iscntrl(ch) && ch < 0x7fu)
			enctype = mime_7bit;
		else if (ch > 0x7fu)
			enctype = mime_8bit;
		else
			enctype = mime_binary;
		
		sprintf(buf, "%s%s", mime_encoding_desc[enctype], ch == 255 ? "" : ",");
		
		fprintf(fp,
			"\t%-40s/* %3d  - %c */\n", buf, ch, isprint(ch) ? ch : '-');
	}
	
	fprintf(fp,
		"};\n"
		"\n"
		"/* end of file */\n");
	
	fclose(fp);
	return 0;			
}
