/*
** mime.h - written by vesely in milano on 26 sep 1999
*/
/*
** Part of MIMEC: see NOTICE.txt
** Copyright (c) 1999-2001 Alessandro Vesely
** All rights reserved.
*/

#include <stdio.h>
/* not #include "parse.h" */

#if !defined(MIME_H_INCLUDED)

/*
** Use module compiled with -DTEST_MAIN to interactively
** test needed functionality. See comments at main() in
** mime.c about debugging functionality.
**
** See comments at exposed functions definitions in mime.c
** for usage comments about each single function.
**
** WARNINGs:
** -careful about field widths when extending enums
** -note that text/plain is the default after memset(,0,)
*/
enum mime_type_enum
{
	mime_text, /* mime_image, mime_audio, mime_video,*/ mime_application,
	mime_multipart /* , mime_message */
};

enum mime_subtype_enum
{
	mime_text_plain, mime_text_other,
	mime_application_octetstream,
	mime_multipart_mixed, mime_multipart_form_data
};

enum mime_encoding_enum /* W: check mime_encoding_desc, do_charencode */
{
	mime_encoding_not_set,
	mime_base_64,
	mime_quoted_printable,
	
	mime_identity,
	mime_7bit = mime_identity,
	mime_8bit,
	mime_binary
};

enum mime_charset_enum
{
	mime_charset_not_set,
	mime_us_ascii,
	mime_iso_8859_1, /* 8859-x, where x = value-1 */
	mime_iso_8859_2,
	mime_iso_8859_3,
	mime_iso_8859_4,
	mime_iso_8859_5,
	mime_iso_8859_6,
	mime_iso_8859_7,
	mime_iso_8859_8,
	mime_iso_8859_9,
	mime_iso_8859_10,
	mime_iso_8859_11,
	mime_iso_8859_12,
	mime_iso_8859_13,
	mime_iso_8859_14,
	mime_iso_8859_15
};

enum mime_disposition_enum
{
	mime_dispo_not_set,
	mime_dispo_is_form_data_field,
	mime_dispo_is_attachment,
	mime_dispo_is_inline
};

/* main structure declaration */
struct mime_tag;
typedef struct mime_tag MIME;
struct mime_clip_tag;
typedef struct mime_clip_tag MIME_CLIP;

/* argument for mime_seek */
typedef enum mime_seek_enum
{
	mime_seek_top,
	mime_seek_downl,
	mime_seek_downr,
	mime_seek_right,
	mime_seek_left,
	mime_seek_up,
	mime_seek_first,
	mime_seek_last,
	mime_seek_next,
	mime_seek_prev
} mime_seek_arg;

/* return code for mime_commit_content */
typedef enum mime_encoding_rtc_enum
{
	mime_ok_7bit, /* mime_identity + result domain */
	mime_ok_8bit,
	mime_ok_binary,
	
	mime_bad_result,
	mime_bad_argument = mime_bad_result,
	mime_bad_structure,
	mime_cannot_do_boundary,
	mime_system_error
} mime_encoding_rtc;

/* argument for mime_set_header */
typedef enum mime_header_enum
{
	mime_version,
	mime_content_type,
	mime_content_disposition,
	mime_content_transfer_encoding,
	mime_content_description,
	mime_content_id,
	
	mime_header_unknown
	
} mime_header_id;

/* last error code */
typedef enum mime_error_enum
{
	MIME_NO_ERROR,                /* all ok */
	MIME_CALL_ERROR,              /* bad call sequence or bad argument(s) */
	MIME_BUFFER_ERROR,            /* special bad call: buffer not enough */
	MIME_DATA_ERROR,              /* bad input data, e.g. boundary mismatch */
	MIME_HEADER_ERROR,            /* couldn't parse header data */
	MIME_INTERNAL_ERROR,          /* e.g. couln't set a boundary */
	MIME_INFILE_ERROR,            /* reading data ferror */
	MIME_FILE_ERROR,              /* internal error handling file content */
	MIME_SYSFILE_ERROR,           /* ferror on I/O files */
	MIME_OVERFLOW_ERROR,          /* max tree capacity exceeded */
	MIME_SYSTEM_ERROR             /* malloc or similar failures */
} mime_error;

/* general functions */
void mime_clear(MIME*);
MIME *mime_new(enum mime_encoding_enum default_encoding,
	enum mime_charset_enum default_charset);
enum mime_charset_enum
	mime_get_default_charset(MIME const*);
enum mime_charset_enum
	mime_set_default_charset(MIME*, enum mime_charset_enum);
mime_error mime_get_last_error(MIME const*);
char const *mime_get_last_error_string(MIME const*);

/* read function */
int mime_read(MIME*, FILE *input_fp);

/* edit and misc functions */
size_t mime_get_count_all(MIME const*);
int mime_is_leaf_part(MIME const*);
int mime_is_empty(MIME const*);
char const * mime_get_name(MIME const*);
char const * mime_get_filename(MIME const*);
char const * mime_get_content_type(MIME const*);
char const * mime_get_boundary(MIME const *);
char const * mime_get_description(MIME const *);
enum mime_type_enum mime_get_mime_type(MIME const *);
enum mime_subtype_enum mime_get_mime_subtype(MIME const *);
enum mime_encoding_enum mime_get_encoding(MIME const*);
enum mime_charset_enum mime_get_charset(MIME const*);
enum mime_disposition_enum mime_get_disposition(MIME const*);

int mime_set_name(MIME*, char const *);
int mime_set_filename(MIME*, char const *);
int mime_set_content_type(MIME*, char const *);
int mime_set_boundary(MIME*, char const *);
int mime_set_description(MIME*, char const *);
enum mime_encoding_enum
	mime_set_encoding(MIME*, enum mime_encoding_enum);
enum mime_charset_enum
	mime_set_charset(MIME*, enum mime_charset_enum);
enum mime_disposition_enum
	mime_set_disposition(MIME*, enum mime_disposition_enum);

size_t mime_read_content(MIME*, void*, size_t, long);
char const * mime_get_content(MIME*, int keep);
unsigned long mime_get_content_length(MIME const*);
size_t mime_get_available_buffer(MIME const*);
int mime_running_out_buffer(MIME const*);
void *mime_detach_buffer(MIME*);
int mime_clear_buffer(MIME*);
int mime_clear_boundary_all(MIME*);
int mime_clear_encoding_all(MIME*);

int mime_clear_content(MIME*);
int mime_add_line_of_content(MIME*, char const *fmt, ...);
int mime_add_text_of_content(MIME*, char const*);
int mime_add_file_of_content(MIME*, FILE*, int is_text);
int mime_attach_binary(MIME*, FILE *);

size_t mime_get_depth(MIME const*);
size_t mime_get_position(MIME const*);
int mime_set_position(MIME*, size_t);
int mime_get_internal_id(MIME const*);
int mime_search_internal_id(MIME*, int);
int mime_seek(MIME *m, mime_seek_arg a);
int mime_seek_is_stuck(MIME const*);
int mime_set_header(MIME*, mime_header_id, const char *);
int mime_mark_for_discard(MIME*);
int mime_commit_discard(MIME*);
int mime_wrap(MIME*);
int mime_insert(MIME*);
int mime_append(MIME*);

/* write functions */
int mime_write(MIME*, FILE *output_fp,
	int unix_line_break, int dot_stuffing);
int mime_extract_single_body(MIME*, FILE*,
	enum mime_encoding_enum, int unix_line_break);
mime_encoding_rtc
	mime_commit_encoding(MIME*, enum mime_encoding_enum);

#define MIME_H_INCLUDED
#endif
