/* parse.h - written by vesely in milan on may 1997 */

/*
** Part of MIMEC: see NOTICE.txt
** Copyright (c) 1999 Alessandro Vesely
** All rights reserved.
*/
#if !defined( PARSE_H_INCLUDED)
#include <stdio.h>

#if defined(__cplusplus)
extern "C" {
#endif

const char *last_sys_errmsg(void); /* return strerror or a static message (errno=xxx) - not NULL */
const char *Psz( const char *p); /* return a non-NULL pointer */
int stricmp( const char *a, const char *b);
int strincmp( const char *a, const char *b, size_t n);
int is_numeric( const char *p);


/* max length of a word -ale:13may2000 was 31, need more for md5 */
#define MAXWORDLENGTH 47


/****************************************************************************/
#if defined( NDEBUG)
#define MEMMOVE_RATIO 16 /* no of bytes I would move to save 1 byte of heap */
#define MAX_READ_SIZE 1024 /* limits the size of memmove */
#define PARSE_BUFFER_SIZE (16*MAX_READ_SIZE) /* limits max size of a string */
#else
#define MEMMOVE_RATIO 8 /* no of bytes I would move to save 1 byte of heap */
#define MAX_READ_SIZE 64 /* limits the size of memmove */
#define PARSE_BUFFER_SIZE 1024 /* limits max size of a string */
#endif

typedef struct parse_tag
{
	char *buf;		/* allocated buffer starts here */
	char *start;	/* next place to store parsed data */
	char *next; 	/* next char to parse */
	char *end;		/* raw memory, not yet read */
	char *endbuf;	/* first byte not usable after buf */

	FILE *fp, *fp_debug;
	char const *fname;
	int line;
	int next_token;
	
	unsigned done:1;
	unsigned no_hash_comment:1;
	unsigned close_file:1;
	
	unsigned int bufndx;
	unsigned int buffer_size; /* size that was malloc'd for buf */
	unsigned long total_size;
	char *bufarray[32]; /* 3lug00:was 16 */
	char word[MAXWORDLENGTH+1];
} PARSE;

int parse_invariant(PARSE *p);
PARSE* parse_init(char const *fname);
PARSE* parse_init_file(FILE* fp, char const *fname,
	unsigned long length, char const *debug_fname);
PARSE *parse_init_memory(char const *buf, char const *fname,
	unsigned long total_size, char const *debug_fname);
void parse_clear( PARSE *p);
int parse_skip( PARSE *p, char *text);
int get_token( PARSE *p);
int get_token_peek(PARSE*); /* don't consume even if punct */
int get_if_delimiter( PARSE *p, char const *delimiters);
int token_must_be(PARSE *p, int tok);
char *get_next( PARSE *p);
char *get_spec( PARSE *p);
char *get_psz_next(PARSE *p);
char *get_word( PARSE *p);
char *get_word_delimited(PARSE *p, char const *delimiters, int, int);
static char*get_any_word(PARSE*p)
	{return get_word_delimited(p, "", 0, 0); }
char *get_text( PARSE *p);
char *get_text_no_special( PARSE *p, char const *delimiters_punct);
char *get_text_including( PARSE *p, char const *alnum_equivalent_punct);
char *get_quoted( PARSE *p);
char *get_possibly_quoted( PARSE *p);
char *get_urlencoded( PARSE *p);
char *get_toeol(PARSE *p);

int get_mime_word(PARSE *p, char **word_ptr, int *deli_ptr);
int get_rfc822_word(PARSE *p, char **word_ptr, int *deli_ptr);
int get_mime_word_relaxed(PARSE *p, char **word_ptr, int *deli_ptr);
typedef struct parse_rfc822_mailbox
{
	unsigned int phrase:1;
	unsigned int literal_route:1;
	unsigned int literal_domain:1;
	int route;
	int route_subdomain_max;
	int subdomain;
	
} parse_rfc822_mailbox;
int skip_rfc822_mailbox(PARSE *p, int *deli_ptr, parse_rfc822_mailbox*);

int copy_until( PARSE *p, FILE *fp, int ch);
char *get_until( PARSE *p, int ch);
char *get_word_until( PARSE *p, int ch);

/* -ale: 28sep99 */
static void parse_set_no_hash_comment(PARSE *p, int new_value)
	{ p->no_hash_comment = new_value; }
static int parse_get_no_hash_comment(PARSE *p)
	{ return p->no_hash_comment; }

#if defined(__cplusplus)
}
#endif

#define PARSE_H_INCLUDED
#endif

